           %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
           %%%    Hybrid Simulation Current Assessment Measures    %%%
           %%%                                                     %%% 
           %%%     Prepared by:                                    %%%
           %%%     Carlos Andres Riascos Gonzalez, Ge(Gaby) Ou,    %%%
           %%%     and Shirley Dyke                                %%%
           %%%                                                     %%%
           %%%    Published on: 10/15/2014                         %%% 
           %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
           
           
function En=cam_integral(t,x,F,method)
% function En=cam_integral(t,x,F,method) evaluates the numerical
% integration of a force-displacement analysis through the following methods:
% 
% - area
% - cumtrapz
% - cycle

% by: Carlos Riascos (August 2014)

if size(F,2)>size(F,1), F=F'; elseif size(x,2)>size(x,1), x=x';end

for m=1:size(F,2)
 
    switch method
        
        case 'cycle'
            
I=find(F(2:end,m).*F(1:end-1,m)<=0); 

if I(end)~=size(F,1),I=[I;size(F,1)]; end
if I(1)~=1,I=[1;I]; end

n=2; E=[]; te=[];

for i=1:n:length(I)-n

    ti=t(I(i):I(i+n));
    xi=[x(I(i):I(i+n),m); x(I(i),m)]; 
    Fi=[F(I(i):I(i+n),m);F(I(i),m)]; 
       
eci=abs(sum(xi(1:end-1).*Fi(2:end))-sum(Fi(1:end-1).*xi(2:end)))/2;
    
   E=[E; eci];
   te=[te; ti(1) ti(end)];

end

En(:,m)=spline(te(:,2),cumsum(E),t);

        case 'cumtrapz'
            
            En(:,m)=cumtrapz(x(:,m),F(:,m));
            
        case 'area'
            
            % Mosqueda et al, 2007

            En(:,m)=cumsum([(F(1:end-1,m)+F(2:end,m)).*(x(2:end,m)-x(1:end-1,m)); 0])/2;
            
            
    end


end