%% Hybrid Simulation CURRENT ASSESSMENT MEASURES (HSCAM)
%%% Hybrid Simulation Current Assessment Measures, Prepared by: Carlos Andres Riascos Gonzalez, Ge(Gaby) Ou, and Shirley Dyke, code published on: 10/15/2014         %
%<<logo4iisl.jpg>>
%
%% 1. Test 
%%    - Dynamics Properties of the Structure
%%    - Time Domain Response 

h0=figure;
set(h0,'units','normalized','position',[0.1 0.1 0.8 0.8])
p=cam_results.parameters;

xc=p.xc; xm=p.xm; Fm=p.fm; t=p.t;
[row,col]=size(Fm);   
aox=1e3;
aof=1e-3; 

no=3;            
                for j=1:col
                
                %%%%%%%%%%%%%%%%%%%%%%%%
                subplot(col,no,no*(j-1)+1)
                %%%%%%%%%%%%%%%%%%%%%%%%

                plot(t,xc(:,j)*aox,'r',t,xm(:,j)*aox,'b--'),ylabel(sprintf('DOF_%d',j)),grid on
                if j==1,title('Displacement(mm)','fontsize',15,'fontname','times new roman'),elseif j==col,xlabel('t(sec)','fontsize',15,'fontname','times new roman'),end
                xlim([t(1) t(end)])
                legend(sprintf('x_c_%d',j),sprintf('x_m_%d',j))
                set(gca,'fontsize',15,'fontname','times new roman')
                
                %%%%%%%%%%%%%%%%%%%%%%%%
                subplot(col,no,no*(j-1)+2)
                %%%%%%%%%%%%%%%%%%%%%%%%

                plot(t,Fm(:,j)*aof,'b--'),grid on
                if j==1,title('Force(kN)','fontsize',15,'fontname','times new roman'),elseif j==col,xlabel('t(sec)','fontsize',15,'fontname','times new roman'),end 
                xlim([t(1) t(end)])
                set(gca,'fontsize',15,'fontname','times new roman')
                legend(sprintf('F_m_%d',j))

                %%%%%%%%%%%%%%%%%%%%%%%%
                subplot(col,no,no*(j-1)+3)
                %%%%%%%%%%%%%%%%%%%%%%%%
                plot(xc(:,j)*aox,Fm(:,j)*aof,'r',xm(:,j)*aox,Fm(:,j)*aof,'b--'),grid on
                if j==1,title('Hysteresis','fontsize',15,'fontname','times new roman'),elseif j==col,xlabel('x(mm)','fontsize',15,'fontname','times new roman'),end
                set(gca,'fontsize',15,'fontname','times new roman')

                end

set(h0,'color','w')
%% 2. Peak Error
% The Peak Error (e) captures the maximum error between the target displacement and the measured
% displacement. eDM is the error between the peak target displacement and the peak measured displacement
% and eMD is the maximum difference error between target and measured displacements. 
%
% <<e.png>>
%

%% 3. Tracking Indicator (TI)
% The Tracking Indicator (TI) is used to quantify the difference between the target and measured 
% displacement of the actuator at each time step. The outcome provides a measure of the error at 
% each time step in the hybrid simulation.
% 
% $$ TI(i+1)=\frac {A(i+1)-TA(i+1)}{2}$$
%
% where:
%
% A(i+1)=A(i)+ \frac {[x_c(i+1)+x_c(i)] [x_m(i+1)+x_m(i)]}{2}
%
%
%
close all
h1=figure;
TI=cam_results.parameters.TI*1e6;
plot(cam_results.parameters.t,TI,'linewidth',2)
hold on
plot(cam_results.parameters.t(end),TI(end,:),'o','linewidth',2)
grid on
xlabel('t(sec)','fontsize',17,'fontname','new times roman')
ylabel('TI(mm^2)','fontsize',17,'fontname','new times roman')
set(gca,'fontsize',15,'fontname','new times roman')
set(gcf,'color','w')
n=size(TI,2);
legend([char(ones(n,1)*['DOF_'] ) num2str([1:n]')],'location','eastoutside')
text(cam_results.parameters.t(end)*ones(1,n),TI(end,:)',num2str(TI(end,:)'),'fontsize',13,'fontname','new times roman')


%% 4. Normalized Root Mean Square Error (NRMSerror)
% The Normalized Root Mean Square (NRMS) can be used to obtain a single value representing this difference 
% between the measured and target displacements of the actuator.  These two measures are time domain 
% measures of the actuator tracking.
%
% $$ NRMS_{error} = \sqrt { \frac {\sum_{i=1}^N [x_m(i)-x_c(i)]^2} {\sum_{i=1}^N x_c(i)^2} } $$
%



%% 5. Frequency Evaluation Index (FEI)
% Frequency Evaluation Index (FEI) is a frequency domain measure of the actuator tracking 
%
% $$ FEI= \sum_{i=i}^N   \frac{fft(x_m)}{fft(x_c)} \frac{|fft(x_m)|^l}{ \sum_{i=i}^N|fft(x_c)|^l}  $$
%
% * Equivalent frequency
%
% $$ f^{eq} = \frac{\sum_{i=i}^N {|fft(x_c)|^l f_j} } {\sum_{i=i}^N {|fft(x_c)|^l} }$$
%
% * Generalized amplitude 
%
% $$ A_0=|FEI|$$
%
% * Generalized angle phase 
%
% $$ \phi=arctan (\frac {Im(FEI)}{Re(FEI)})$$
%
% * Equivalent delay 
%
% $$ \delta=-\frac {\phi}{2\pi f^{eq}}$$
%
h4=figure;
n=length(cam_results.FEI);
col='bgrcmyk';
for i=1:n
k=rem(i,7); if k==0, k=7; end 
Com(i)=compass(cam_results.FEI(i),col(k)); hold on
end
set(gca,'fontsize',15,'fontname','new times roman')
set(Com,'linewidth',2)
legend([char(ones(n,1)*['DOF_'] ) num2str([1:n]')],'location','eastoutside')
set(h4,'color','w')
xlabel('Real Axis','fontsize',15,'fontname','new times roman')
ylabel('Imag Axis','fontsize',15,'fontname','new times roman')
title('FEI','fontsize',17,'fontname','new times roman')

%% 6. Cross Correlation
% The Cross Correlation (CC) between the target and measured displacement can also provide 
% frequency domain insight into the performance of actuator tracking. Cross-correlation quantifies 
% the degree of similarity between two time series. In hybrid simulation, cross-correlation between
% the target and measured displacements provides a reasonable estimate of time delay in the actuator. 
%
% $$[x_c*x_m](\tau) = \int_{-\infty}^{\infty} x_c(\tau)x_m(t-\tau) d\tau $$
%
% $$ \tau_{est}=argmax{[x_c*x_m](\tau)} $$
%

h2=figure;
n=size(cam_results.parameters.xcXxm,2);

for i=1:n
subplot(n,1,i) 
plot(cam_results.parameters.lags(:,i),cam_results.parameters.xcXxm(:,i),'linewidth',1)
hold on
plot(cam_results.parameters.lags(cam_results.parameters.CCI(i),i),cam_results.parameters.xcXxm(cam_results.parameters.CCI(i),i),'ro','linewidth',2)
grid on
hold off

ylabel(['x_c_',num2str(i),'*x_m_',num2str(i)],'fontsize',17,'fontname','new times roman')
set(gca,'fontsize',15,'fontname','new times roman')
text(cam_results.parameters.lags(cam_results.parameters.CCI(i),i),cam_results.parameters.xcXxm(cam_results.parameters.CCI(i),i),num2str(cam_results.parameters.lags(cam_results.parameters.CCI(i),i)),'fontsize',13,'fontname','new times roman')

end

set(gcf,'color','w')
xlabel('\tau','fontsize',17,'fontname','new times roman')

%% 7. Energy Error
% Mosqueda et al. (2007) proposed the use of energy to quantify the difference between actual 
% experimental behavior (measured forces versus measured displacements) and that observed
% in explicit numerical simulations (measured forces versus target displacements). 
%
% $$ E_E^{err}=E_E^c-E_E $$
%

h3=figure;

n=size(cam_results.parameters.Eerr_exp,2);
p=cam_results.parameters;

for i=1:n

%%%%%%%%%%%%%%%%%%%%%%
subplot(n,3,3*(i-1)+[1 2])
%%%%%%%%%%%%%%%%%%%%%%
EE2=p.EK(:,i)+p.ED(:,i)+p.ES(:,i)+p.EEc(:,i);
plot(p.t,p.EEc(:,i),'r',p.t,p.EE(:,i),'b','linewidth',2)
%plot(p.t,p.EEc(:,i),'r',p.t,p.EE(:,i),'b',p.t,EE2,'k--','linewidth',2)
grid on, set(gca,'fontsize',13,'fontname','new times roman')
xlim([p.t(1) p.t(end)])
ylabel(['DOF_',num2str(i)],'fontsize',13,'fontname','new times roman')
%legend(['E_E^c_',num2str(i)],['E_E_',num2str(i)],'E_E''','location','best')
%legend('E_E^c','E_E','location','northoutside','orientation','horizontal')

ylim([floor(min(min([p.EEc(:,i) p.EE(:,i)]))) ceil(max(max([p.EEc(:,i) p.EE(:,i)]))) ])
if i==n, xlabel('t(sec)','fontsize',17,'fontname','new times roman'),end

%%%%%%%%%%%%%%%%%%%%%%
subplot(n,3,3*(i-1)+3)
%%%%%%%%%%%%%%%%%%%%%%

plot(cam_results.parameters.t,cam_results.parameters.Eerr_exp(:,i),'k','linewidth',2)
hold on
plot(p.t(end),p.Eerr_exp(end,i),'ko','linewidth',2)
grid on, set(gca,'fontsize',15,'fontname','new times roman')
text(p.t(end),p.Eerr_exp(end,i),num2str(p.Eerr_exp(end,i)'),'fontsize',13,'fontname','new times roman')
%ylabel(['E_e_r_r_',num2str(i),'^e^x^p(J)'],'fontsize',17,'fontname','new times roman')
xlim([p.t(1) p.t(end)])

if i==1, title(['E_e_r_r^e^x^p(J)'],'fontsize',17,'fontname','new times roman'),end
if i==n, xlabel('t(sec)','fontsize',17,'fontname','new times roman'),end

end

set(gcf,'color','w')
%% 8. Hybrid Simulation Energy Monitor (HSEM)
% Mosqueda et al. (2007b) showed that one can limit the displacement and force errors of a hybrid simulation
% by limiting the amount of the HSEM. Since the majority of errors in a hybrid simulation are likely from 
% experimental sources, the HSEM can be a suitable choice for monitoring the simulation quality. 
%
% $$ HSEM = \frac{E_E^{err}}{E_I-E_E^{max}} $$
% 
% where:
%
% $$ E_I=\int F_m^T dx $$
%
% $$ E_E^{err}=\frac {1}{2}x_0^TK^0x_0 $$
%
h5=figure;

p=cam_results.parameters;
n=size(p.HSEM,2);

plot(p.t,p.HSEM,'linewidth',2),grid on

xlabel('t(sec)','fontsize',17,'fontname','times new roman')
title('HSEM','fontsize',17,'fontname','times new roman')
legend([char(ones(n,1)*['DOF_'] ) num2str([1:n]')],'location','eastoutside')

xlim([t(1) t(end)])

set(gca,'fontsize',15,'fontname','times new roman')

set(h5,'color','w')
%% 9. Nomenclature
%
% <<nomenclature.png>>
%